﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Net;
using System.IO;
using System.Text;

namespace Unibase
{
    public class JsonParkingParser : IDisposable
    {
        private List<Parking> parkings;
        private HttpWebRequest req;
        private HttpWebResponse resp;
        StreamReader reader;

        public JsonParkingParser()
        {
            parkings = new List<Parking>();
        }

        public List<Parking> ParseUrl(string url)
        {
            string json = "";
            Encoding enc = Encoding.UTF8;

            //Create a new WebRequest object with the url for the service
            req = WebRequest.Create(url) as HttpWebRequest;
            resp = req.GetResponse() as HttpWebResponse;
     
            reader = new StreamReader(resp.GetResponseStream(), enc);

            //Reads the json-response from the service
            json = reader.ReadToEnd();

            //Return an empty list if no data is returned
            if (json == "[]")
                return new List<Parking>();

            //Remove unneccesary characters
            json = json.Remove(0, 2);
            json = json.Remove(json.Length - 2);

            //Split the json-response into an array of parkings
            string[] parkings = json.Split('}');

            for (int i = 0; i < parkings.Length; i++)
            {
                parkings[i] = parkings[i].Replace(",{", "");
            }

            System.Globalization.CultureInfo ci = System.Globalization.CultureInfo.CreateSpecificCulture("sv-SE");

            foreach (string s in parkings)
            {
                
                //Parse the values for the current parking
                string ID = GetValue("Id", s);
                string parkableLength = GetValue("ParkableLength", s);
                string sitename = GetValue("Name", s);
                string maxParkingTime = GetValue("MaxParkingTime", s);
                string latitude = GetValue("Lat", s);
                string longitude = GetValue("Long", s);
                string parkingSpaces = GetValue("ParkingSpaces", s);
                string parkingCost = GetValue("ParkingCost", s);
                string parkingCharge = GetValue("ParkingCharge", s);
                string currentParkingCost = GetValue("CurrentParkingCost", s);
                string extraInfo = GetValue("ExtraInfo", s);
                string owner = GetValue("Owner", s);
                string freeSpaces = GetValue("FreeSpaces", s);
                string freeSpacesDate = GetValue("FreeSpacesDate", s);
                string parkingSpaceCount = GetValue("ParkingSpaceCount", s);
                string residentialParkingArea = GetValue("ResidentialParkingArea", s);
                string distance = GetValue("Distance", s);

                //Create a new parking object with the parsed data
                Parking p = new Parking();
                p.CurrentParkingCost = currentParkingCost != "" ? Int32.Parse(currentParkingCost) : (int?)null;
                p.Distance = distance != "" ? Int32.Parse(distance) : (int?)null;
                p.ExtraInfo = extraInfo;
                p.FreeSpaces = freeSpaces != "" ? Int32.Parse(freeSpaces) : (int?)null;
                if (freeSpacesDate != "")
                {
                    freeSpacesDate = freeSpacesDate.Substring(freeSpacesDate.IndexOf("Date(") + 5);
                    freeSpacesDate = freeSpacesDate.Substring(0, freeSpacesDate.IndexOf("+"));
                    DateTime unixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
                    DateTime dotnetTime = unixEpoch.AddMilliseconds(Convert.ToDouble(freeSpacesDate));
                    p.FreeSpacesDate = dotnetTime.ToLocalTime();
                }
                p.Latitude = double.Parse(latitude.Replace(".", ","), System.Globalization.NumberStyles.Any, ci);
                p.Longitude = double.Parse(longitude.Replace(".", ","), System.Globalization.NumberStyles.Any, ci);
                p.MaxParkingTime = maxParkingTime;
                p.ParkableLength = parkableLength != "" ? Int32.Parse(parkableLength) : (int?)null;
                p.ParkingCost = parkingCost;
                p.ParkingSpaceCount = parkingSpaceCount != "" ? Int32.Parse(parkingSpaceCount) : (int?)null;
                p.ParkningSpaces = parkingSpaces != "" ? Int32.Parse(parkingSpaces) : (int?)null;
                p.ResidentialParking = residentialParkingArea;
                p.Name = sitename;
                p.Id = ID;
                p.ParkingCharge = parkingCharge;
                p.Owner = owner;

                this.parkings.Add(p);
            }

            return this.parkings;
        }

        private string GetValue(string property, string parking)
        {
            string value = string.Empty;

            //Check if parking has the specified property
            if (parking.IndexOf(property) > 0)
            {
                value = parking.Substring(parking.IndexOf(property));

                int length = 0;

                //Check if it is a number or string
                if (value.IndexOf(property + "\":\"") == -1)
                    length = value.IndexOf(",");
                else
                    length = value.IndexOf("\",");

                if (length <= 0)
                    length = value.Length;

                value = value.Substring(0, length);
                value = value.Substring(value.IndexOf(":") + 1);

                value = value.Replace("\"", "");
                value = value.Replace(@"\", "");
            }
            return value;
        }

        #region IDisposable Members

        public void Dispose()
        {
            this.parkings = null;
            this.reader.Dispose();
            this.req = null;
            this.resp = null;
        }

        #endregion
    }
}