﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Services;
using Unibase;
using System.Configuration;
using System.Globalization;
using System.Text;

public partial class _Default : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        //Make sure to set your data.goteborg.se userkey in web.config
        if (string.IsNullOrEmpty(ConfigurationManager.AppSettings["WSKey"]))
            throw new Exception("Key for data.goteborg.se must be set in web.config. You can get your own key at http://data.goteborg.se/Account/Register.aspx");
    }

    [WebMethod()]
    public static object GetGoogleMapsJS(string latitude, string longitude, string radius, string type)
    {
        string url = "", key = "", pin = "", marker = "";
        double lat, lon;
        int rad, counter = 1;

        key = ConfigurationManager.AppSettings["WSKey"];

        if (!double.TryParse(latitude, NumberStyles.Any, new CultureInfo("en-US"), out lat))
            return new { Error = true, Message = "alert('Latitude is not correctly formatted.');" };
        else if (!double.TryParse(longitude, NumberStyles.Any, new CultureInfo("en-US"), out lon))
            return new { Error = true, Message = "alert('Longitude is not correctly formatted.);" };
        else if (!Int32.TryParse(radius, out rad))
            return new { Error = true, Message = "alert('Radius is not correctly formatted.');" };

        //Sets the correct url based on user selection
        switch (type)
        {
            case "private":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/PrivateParkings/{0}?format=json", key);
                break;
            case "time":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/PublicTimeParkings/{0}?format=json", key);
                break;
            case "toll":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/PublicTollParkings/{0}?format=json", key);
                break;
            case "mc":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/MCParkings/{0}?format=json", key);
                break;
            case "bus":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/BusParkings/{0}?format=json", key);
                break;
            case "residential":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/ResidentialParkings/{0}?format=json", key);
                break;
            case "handicap":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/HandicapParkings/{0}?format=json", key);
                break;
            case "truck":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/TruckParkings/{0}?format=json", key);
                break;
            case "pay":
                url = string.Format("http://data.goteborg.se/ParkingService/v1.0/PublicPayMachines/{0}?format=json", key);
                break;
        }

        //If coordinates in supplied, add these to the webservice url.
        if (lat > 0 && lon > 0)
            url += string.Format("&latitude={0}&longitude={1}&radius={2}", lat.ToString(new CultureInfo("en-US")), lon.ToString(new CultureInfo("en-US")), rad);

        JsonParkingParser parser = new JsonParkingParser();
        List<Parking> parkings = parser.ParseUrl(url);
        parser.Dispose();

        StringBuilder sbJavascript = new StringBuilder();
        //Create a javascript for all pins returned from the webservice and place these on the map
        foreach (Parking p in parkings)
        {
            pin = string.Format("pin{0}", counter);
            marker = string.Format("new google.maps.LatLng({0}, {1})", p.Latitude.ToString(new CultureInfo("en-US")), p.Longitude.ToString(new CultureInfo("en-US")));

            sbJavascript.Append("var ");
            sbJavascript.Append(pin);
            sbJavascript.Append(" = new google.maps.Marker({ position:");
            sbJavascript.Append(marker);
            sbJavascript.Append(", title:\"");
            sbJavascript.Append(p.Name);
            sbJavascript.AppendLine("\"});");

            sbJavascript.AppendLine("info = new google.maps.InfoWindow();");

            sbJavascript.Append("pins.push(");
            sbJavascript.Append(pin);
            sbJavascript.AppendLine(");");

            sbJavascript.Append("google.maps.event.addListener(");
            sbJavascript.Append(pin);
            sbJavascript.Append(", \"mouseover\", function() { info.setContent('");
            sbJavascript.Append(p.Name);
            sbJavascript.Append("<br/>");
            sbJavascript.Append(p.ParkingCost);
            sbJavascript.AppendLine("');");

            sbJavascript.Append("info.open(map, ");
            sbJavascript.Append(pin);
            sbJavascript.AppendLine("); });");

            sbJavascript.Append(pin);
            sbJavascript.AppendLine(".setMap(map);");

            counter++;
        }

        return new { Error = false, Message = sbJavascript.ToString() };
    }
}